@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2009 GraphicsMagick Group
+% Copyright (C) 2003-2011 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -40,6 +40,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/analyze.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/colormap.h"
@@ -137,8 +138,12 @@ typedef struct _ErrorManager
   Image
     *image;
 
+  MagickBool
+    completed;
+
   jmp_buf
     error_recovery;
+
 } ErrorManager;
 
 typedef struct _SourceManager
@@ -189,31 +194,62 @@ typedef struct _SourceManager
 %
 */
 
-static unsigned int EmitMessage(j_common_ptr jpeg_info,int level)
+/*
+  Format a libjpeg warning or trace event.  Warnings are converted to
+  GraphicsMagick warning exceptions while traces are optionally
+  logged.
+*/
+static unsigned int JPEGMessageHandler(j_common_ptr jpeg_info,int msg_level)
 {
   char
     message[JMSG_LENGTH_MAX];
 
+  struct jpeg_error_mgr
+    *err;
+
   ErrorManager
     *error_manager;
 
   Image
     *image;
 
-  (jpeg_info->err->format_message)(jpeg_info,message);
+  message[0]='\0';
+  err=jpeg_info->err;
   error_manager=(ErrorManager *) jpeg_info->client_data;
   image=error_manager->image;
-  if (level < 0)
+  /* msg_level is -1 for warnings, 0 and up for trace messages. */
+  if (msg_level < 0)
     {
-      if ((jpeg_info->err->num_warnings == 0) ||
-          (jpeg_info->err->trace_level >= 3))
-        ThrowBinaryException2(CorruptImageWarning,(char *) message,
-          image->filename);
-      jpeg_info->err->num_warnings++;
+      /* A warning */
+      (err->format_message)(jpeg_info,message);
+
+      if (image->logging)
+	{
+	  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+				"[%s] JPEG Warning: \"%s\" (code=%d, parms=0x%02x,0x%02x,"
+				"0x%02x,0x%02x,0x%02x,0x%02x,0x%02x,0x%02x)",
+				image->filename,message,err->msg_code,
+				err->msg_parm.i[0], err->msg_parm.i[1],
+				err->msg_parm.i[2], err->msg_parm.i[3],
+				err->msg_parm.i[4], err->msg_parm.i[5],
+				err->msg_parm.i[6], err->msg_parm.i[7]);
+	}
+      if ((err->num_warnings == 0) ||
+          (err->trace_level >= 3))
+	ThrowBinaryException2(CorruptImageWarning,(char *) message,
+			      image->filename);
+      err->num_warnings++;
     }
   else
-    if (jpeg_info->err->trace_level >= level)
-      ThrowBinaryException2(CoderError,(char *) message,image->filename);
+    {
+      /* A trace message */
+      if ((image->logging) && (msg_level >= err->trace_level))
+	{
+	  (err->format_message)(jpeg_info,message);
+	  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+				"[%s] JPEG Trace: \"%s\"",image->filename,message);
+	}
+    }
   return(True);
 }
 
@@ -256,13 +292,47 @@ static void InitializeSource(j_decompress_ptr cinfo)
   source->start_of_blob=TRUE;
 }
 
+/*
+  Format and report a libjpeg error event.  Errors are reported via a
+  GraphicsMagick error exception. The function terminates with
+  longjmp() so it never returns to the caller.
+*/
+static void JPEGErrorHandler(j_common_ptr jpeg_info) MAGICK_FUNC_NORETURN;
+
 static void JPEGErrorHandler(j_common_ptr jpeg_info)
 {
+  char
+    message[JMSG_LENGTH_MAX];
+
+  struct jpeg_error_mgr
+    *err;
+
   ErrorManager
     *error_manager;
 
-  (void) EmitMessage(jpeg_info,0);
-  error_manager=( ErrorManager *) jpeg_info->client_data;
+  Image
+    *image;
+
+  message[0]='\0';
+  err=jpeg_info->err;
+  error_manager=(ErrorManager *) jpeg_info->client_data;
+  image=error_manager->image;
+  (err->format_message)(jpeg_info,message);
+  if (image->logging)
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+			    "[%s] JPEG Error: \"%s\" (code=%d, parms=0x%02x,0x%02x,"
+			    "0x%02x,0x%02x,0x%02x,0x%02x,0x%02x,0x%02x)",
+			    image->filename,message,err->msg_code,
+			    err->msg_parm.i[0], err->msg_parm.i[1],
+			    err->msg_parm.i[2], err->msg_parm.i[3],
+			    err->msg_parm.i[4], err->msg_parm.i[5],
+			    err->msg_parm.i[6], err->msg_parm.i[7]);
+    }
+  if (error_manager->completed)
+    ThrowException2(&image->exception,CoderWarning,(char *) message,image->filename);
+  else
+    ThrowException2(&image->exception,CoderError,(char *) message,image->filename);
   longjmp(error_manager->error_recovery,1);
 }
 
@@ -505,47 +575,44 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
 #endif
   error_manager=(ErrorManager *) jpeg_info->client_data;
   image=error_manager->image;
-  if (GetImageProfile(image,"IPTC",(size_t *) NULL) == 0)
-    {
 #ifdef GET_ONLY_IPTC_DATA
-      /*
-        Find the beginning of the IPTC portion of the binary data.
-      */
-      for (*tag='\0'; length > 0; )
-      {
-        *tag=GetCharacter(jpeg_info);
-        *(tag+1)=GetCharacter(jpeg_info);
-        length-=2;
-        if ((*tag == 0x1c) && (*(tag+1) == 0x02))
-          break;
-      }
-      tag_length=2;
-#else
-      /*
-        Validate that this was written as a Photoshop resource format slug.
-      */
-      for (i=0; i < 10; i++)
-        magick[i]=GetCharacter(jpeg_info);
-      magick[10]='\0';
-      length-=10;
-      if (LocaleCompare(magick,"Photoshop ") != 0)
-        {
-          /*
-            Not a ICC profile, return.
-          */
-          for (i=0; i < length; i++)
-            (void) GetCharacter(jpeg_info);
-          return(True);
-        }
-      /*
-        Remove the version number.
-      */
-      for (i=0; i < 4; i++)
-        (void) GetCharacter(jpeg_info);
-      length-=4;
-      tag_length=0;
-#endif
+  /*
+    Find the beginning of the IPTC portion of the binary data.
+  */
+  for (*tag='\0'; length > 0; )
+    {
+      *tag=GetCharacter(jpeg_info);
+      *(tag+1)=GetCharacter(jpeg_info);
+      length-=2;
+      if ((*tag == 0x1c) && (*(tag+1) == 0x02))
+	break;
     }
+  tag_length=2;
+#else
+  /*
+    Validate that this was written as a Photoshop resource format slug.
+  */
+  for (i=0; i < 10; i++)
+    magick[i]=GetCharacter(jpeg_info);
+  magick[10]='\0';
+  length-=10;
+  if (LocaleCompare(magick,"Photoshop ") != 0)
+    {
+      /*
+	Not a ICC profile, return.
+      */
+      for (i=0; i < length; i++)
+	(void) GetCharacter(jpeg_info);
+      return(True);
+    }
+  /*
+    Remove the version number.
+  */
+  for (i=0; i < 4; i++)
+    (void) GetCharacter(jpeg_info);
+  length-=4;
+  tag_length=0;
+#endif
   if (length <= 0)
     return(True);
 
@@ -588,7 +655,7 @@ static void SkipInputData(j_decompress_ptr cinfo,long number_bytes)
 
 static void TerminateSource(j_decompress_ptr cinfo)
 {
-  cinfo=cinfo;
+  (void) cinfo;
 }
 
 static void JPEGSourceManager(j_decompress_ptr cinfo,Image *image)
@@ -868,6 +935,33 @@ FormatJPEGSamplingFactors(const struct jpeg_decompress_struct *jpeg_info,
     }
 }
 
+static MagickBool
+IsITUFax(const Image* image)
+{
+  size_t
+    profile_length;
+  
+  const unsigned char
+    *profile;
+  
+  MagickBool
+    status;
+  
+  status=MagickFalse;
+  if ((profile=GetImageProfile(image,"APP1",&profile_length)) &&
+      (profile_length >= 5))
+    {
+      if (profile[0] == 0x47 &&
+	  profile[1] == 0x33 &&
+	  profile[2] == 0x46 &&
+	  profile[3] == 0x41 &&
+	  profile[4] == 0x58)
+      status=MagickTrue;
+    }
+
+    return status;
+}
+
 static Image *ReadJPEGImage(const ImageInfo *image_info,
 			    ExceptionInfo *exception)
 {
@@ -916,18 +1010,18 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if (image == (Image *) NULL)
-    {
-      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
-			   image);
-    }
+    ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFail)
     ThrowReaderException(FileOpenError,UnableToOpenFile,image);
   /*
-    Initialize image structure.
+    Initialize structures.
   */
+  (void) memset(&error_manager,0,sizeof(error_manager));
+  (void) memset(&jpeg_info,0,sizeof(jpeg_info));
+  (void) memset(&jpeg_error,0,sizeof(jpeg_error));
   jpeg_info.err=jpeg_std_error(&jpeg_error);
-  jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) EmitMessage;
+  jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) JPEGMessageHandler;
   jpeg_info.err->error_exit=(void (*)(j_common_ptr)) JPEGErrorHandler;
   jpeg_pixels=(JSAMPLE *) NULL;
   error_manager.image=image;
@@ -946,6 +1040,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       return((Image *) NULL);
     }
   jpeg_info.client_data=(void *) &error_manager;
+
   jpeg_create_decompress(&jpeg_info);
   JPEGSourceManager(&jpeg_info,image);
   jpeg_set_marker_processor(&jpeg_info,JPEG_COM,ReadComment);
@@ -955,8 +1050,21 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     if ((i != 2) && (i != 13) && (i != 14))
       jpeg_set_marker_processor(&jpeg_info,JPEG_APP0+i,ReadGenericProfile);
   i=jpeg_read_header(&jpeg_info,True);
-  if (jpeg_info.out_color_space == JCS_CMYK)
-    image->colorspace=CMYKColorspace;
+  if (IsITUFax(image))
+    {
+      if (image->logging)
+	(void) LogMagickEvent(CoderEvent,GetMagickModule(),
+			      "Image colorspace set to LAB");
+      image->colorspace=LABColorspace;
+      jpeg_info.out_color_space = JCS_YCbCr;
+    }
+  else if (jpeg_info.out_color_space == JCS_CMYK)
+    {
+      if (image->logging)
+	(void) LogMagickEvent(CoderEvent,GetMagickModule(),
+			      "Image colorspace set to CMYK");
+      image->colorspace=CMYKColorspace;
+    }
   if (jpeg_info.saw_JFIF_marker)
     {
       if ((jpeg_info.X_density != 1U) && (jpeg_info.Y_density != 1U))
@@ -972,32 +1080,60 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
             image->units=PixelsPerCentimeterResolution;
         }
     }
+  /*
+    If the desired image size is pre-set (e.g. by using -size), then
+    let the JPEG library subsample for us.
+  */
   number_pixels=image->columns*image->rows;
   if (number_pixels != 0)
     {
       double
         scale_factor;
 
-      /*
-        Let the JPEG library subsample for us.
-      */
+
+      if (image->logging)
+	(void) LogMagickEvent(CoderEvent,GetMagickModule(),
+			      "Requested Geometry: %lux%lu",
+			      image->columns,image->rows);
       jpeg_calc_output_dimensions(&jpeg_info);
       image->magick_columns=jpeg_info.output_width;
       image->magick_rows=jpeg_info.output_height;
       scale_factor=(double) jpeg_info.output_width/image->columns;
       if (scale_factor > ((double) jpeg_info.output_height/image->rows))
         scale_factor=(double) jpeg_info.output_height/image->rows;
-      jpeg_info.scale_denom=(unsigned int) scale_factor;
+      jpeg_info.scale_denom *=(unsigned int) scale_factor;
       jpeg_calc_output_dimensions(&jpeg_info);
       if (image->logging)
 	(void) LogMagickEvent(CoderEvent,GetMagickModule(),
-			      "Scale_factor: %ld",(long) scale_factor);
+			      "Scale_factor: %ld (scale_num=%d, "
+			      "scale_denom=%d)",
+			      (long) scale_factor,
+			      jpeg_info.scale_num,jpeg_info.scale_denom);
     }
-  if (image_info->subrange != 0)
+#if 0
+  /*
+    The subrange parameter is set by the filename array syntax similar
+    to the way an image is requested from a list (e.g. myfile.jpg[2]).
+    Argument values other than zero are used to scale the image down
+    by that factor.  IJG JPEG 62 (6b) supports values of 1,2,4, or 8
+    while IJG JPEG 70 supports all values in the range 1-16.  This
+    feature is useful in case you want to view all of the images with
+    a consistent ratio.  Unfortunately, it uses the same syntax as
+    list member access.
+  */
+  else if (image_info->subrange != 0)
     {
-      jpeg_info.scale_denom=(int) image_info->subrange;
+      jpeg_info.scale_denom *=(int) image_info->subrange;
       jpeg_calc_output_dimensions(&jpeg_info);
+      if (image->logging)
+	(void) LogMagickEvent(CoderEvent,GetMagickModule(),
+			      "Requested Scaling Denominator: %d "
+			      "(scale_num=%d, scale_denom=%d)",
+			      (int) image_info->subrange,
+			      jpeg_info.scale_num,jpeg_info.scale_denom);
+
     }
+#endif
 #if (JPEG_LIB_VERSION >= 61) && defined(D_PROGRESSIVE_SUPPORTED)
 #ifdef D_LOSSLESS_SUPPORTED
   image->interlace=
@@ -1016,6 +1152,49 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   image->compression=JPEGCompression;
   image->interlace=LineInterlace;
 #endif
+  {
+    const char
+      *value;
+
+    /*
+      Allow the user to enable/disable block smoothing.
+    */
+    if ((value=AccessDefinition(image_info,"jpeg","block-smoothing")))
+      {
+	if (LocaleCompare(value,"FALSE") == 0)
+          jpeg_info.do_block_smoothing=False;
+        else
+          jpeg_info.do_block_smoothing=True;
+      }
+
+    /*
+      Allow the user to select the DCT decoding algorithm.
+    */
+    if ((value=AccessDefinition(image_info,"jpeg","dct-method")))
+      {
+        if (LocaleCompare(value,"ISLOW") == 0)
+          jpeg_info.dct_method=JDCT_ISLOW;
+        else if (LocaleCompare(value,"IFAST") == 0)
+          jpeg_info.dct_method=JDCT_IFAST;
+        else if (LocaleCompare(value,"FLOAT") == 0)
+          jpeg_info.dct_method=JDCT_FLOAT;
+        else if (LocaleCompare(value,"DEFAULT") == 0)
+          jpeg_info.dct_method=JDCT_DEFAULT;
+        else if (LocaleCompare(value,"FASTEST") == 0)
+          jpeg_info.dct_method=JDCT_FASTEST;
+      }
+
+    /*
+      Allow the user to enable/disable fancy upsampling.
+    */
+    if ((value=AccessDefinition(image_info,"jpeg","fancy-upsampling")))
+      {
+	if (LocaleCompare(value,"FALSE") == 0)
+          jpeg_info.do_fancy_upsampling=False;
+        else
+          jpeg_info.do_fancy_upsampling=True;
+      }
+  }
   (void) jpeg_start_decompress(&jpeg_info);
   image->columns=jpeg_info.output_width;
   image->rows=jpeg_info.output_height;
@@ -1034,6 +1213,12 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Geometry: %dx%d",
 			    (int) jpeg_info.output_width,
 			    (int) jpeg_info.output_height);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"DCT Method: %d",
+			    jpeg_info.dct_method);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Fancy Upsampling: %s",
+			    (jpeg_info.do_fancy_upsampling ? "true" : "false"));
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Block Smoothing: %s",
+			    (jpeg_info.do_block_smoothing ? "true" : "false"));
     }
 
   {
@@ -1053,7 +1238,8 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     (void) SetImageAttribute(image,"JPEG-Colorspace-Name",attribute);
     if (image->logging)
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-			    "Colorspace: %s", attribute);
+			    "Colorspace: %s (%d)", attribute,
+			    jpeg_info.out_color_space);
 
     FormatJPEGSamplingFactors(&jpeg_info,attribute);
     (void) SetImageAttribute(image,"JPEG-Sampling-factors",attribute);
@@ -1119,9 +1305,11 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
 	}
       indexes=AccessMutableIndexes(image);
 
-      /* FIXME .... */
       if (jpeg_read_scanlines(&jpeg_info,scanline,1) != 1)
-	ThrowReaderException(CorruptImageError,CorruptImage,image);
+	{
+	  status=MagickFail;
+	  break;
+	}
 
       p=jpeg_pixels;
 
@@ -1192,6 +1380,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
 				    image->columns,image->rows))
 	  {
 	    status=MagickFail;
+	    jpeg_abort_decompress(&jpeg_info);
 	    break;
 	  }
     }
@@ -1199,7 +1388,20 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     Free jpeg resources.
   */
   if (status == MagickPass)
-    (void) jpeg_finish_decompress(&jpeg_info);
+    {
+      /*
+	jpeg_finish_decompress() may throw an exception while it is
+	finishing the remainder of the JPEG file.  At this point we
+	have already decoded the image so we handle exceptions from
+	jpeg_finish_decompress() specially, mapping reported
+	exceptions as warnings rather than errors.  We try using
+	jpeg_finish_decompress() and if it results in a longjmp(),
+	then we skip over it again.
+      */
+      error_manager.completed=MagickTrue;
+      if (!setjmp(error_manager.error_recovery))
+	(void) jpeg_finish_decompress(&jpeg_info);
+    }
   jpeg_destroy_decompress(&jpeg_info);
   MagickFreeMemory(jpeg_pixels);
   CloseBlob(image);
@@ -1223,7 +1425,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
 	  int
 	    orientation;
 
-	  orientation=atoi(attribute->value);
+	  orientation=MagickAtoI(attribute->value);
 	  if ((orientation > UndefinedOrientation) &&
 	      (orientation <= LeftBottomOrientation))
 	    image->orientation=(OrientationType) orientation;
@@ -1389,29 +1591,6 @@ static void InitializeDestination(j_compress_ptr cinfo)
   destination->manager.free_in_buffer=MaxBufferExtent;
 }
 
-static unsigned int JPEGWarningHandler(j_common_ptr jpeg_info,int level)
-{
-  char
-    message[JMSG_LENGTH_MAX];
-
-  Image
-    *image;
-
-  (jpeg_info->err->format_message)(jpeg_info,message);
-  image=(Image *) jpeg_info->client_data;
-  if (level < 0)
-    {
-      if ((jpeg_info->err->num_warnings == 0) ||
-          (jpeg_info->err->trace_level >= 3))
-        ThrowBinaryException2(CoderError,(char *) message,image->filename);
-      jpeg_info->err->num_warnings++;
-    }
-  else
-    if (jpeg_info->err->trace_level >= level)
-      ThrowBinaryException2(CoderError,(char *) message,image->filename);
-  return(True);
-}
-
 static void TerminateDestination(j_compress_ptr cinfo)
 {
   DestinationManager
@@ -1473,8 +1652,9 @@ static void WriteICCProfile(j_compress_ptr jpeg_info,Image *image)
     jpeg_write_marker(jpeg_info,ICC_MARKER,profile,(unsigned int) length+14);
     MagickFreeMemory(profile);
   }
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "ICC profile: %ld bytes",(long) profile_length);
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+			  "ICC profile: %ld bytes",(long) profile_length);
 }
 
 static void WriteIPTCProfile(j_compress_ptr jpeg_info,Image *image)
@@ -1537,8 +1717,9 @@ static void WriteIPTCProfile(j_compress_ptr jpeg_info,Image *image)
     MagickFreeMemory(profile);
   }
 
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                        "IPTC profile: %ld bytes",(long) profile_length);
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+			  "IPTC profile: %ld bytes",(long) profile_length);
 }
 
 static void JPEGDestinationManager(j_compress_ptr cinfo,Image * image)
@@ -1557,6 +1738,9 @@ static void JPEGDestinationManager(j_compress_ptr cinfo,Image * image)
 
 static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
 {
+  ErrorManager
+    error_manager;
+
   const ImageAttribute
     *attribute;
 
@@ -1614,6 +1798,16 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
     ThrowWriterException(FileOpenError,UnableToOpenFile,image);
+
+  (void) memset(&error_manager,0,sizeof(error_manager));
+  (void) memset(&jpeg_info,0,sizeof(jpeg_info));
+  (void) memset(&jpeg_error,0,sizeof(jpeg_error));
+
+  /*
+    Transform image to user-requested colorspace.
+  */
+  if (UndefinedColorspace != image_info->colorspace)
+    (void) TransformColorspace(image,image_info->colorspace);
   
   /*
     Analyze image to be written.
@@ -1627,58 +1821,78 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
     }
 
   /*
-    Initialize JPEG parameters.
+    Set initial longjmp based error handler.
   */
-  (void) memset(&jpeg_info,0,sizeof(jpeg_info));
-  (void) memset(&jpeg_error,0,sizeof(jpeg_error));
   jpeg_info.client_data=(void *) image;
   jpeg_info.err=jpeg_std_error(&jpeg_error);
-  jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) JPEGWarningHandler;
+  jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) JPEGMessageHandler;
+  jpeg_info.err->error_exit=(void (*)(j_common_ptr)) JPEGErrorHandler;
+  error_manager.image=image;
+  if (setjmp(error_manager.error_recovery))
+    {
+      jpeg_destroy_compress(&jpeg_info);
+      CloseBlob(image);
+      return MagickFail ;
+    }
+  jpeg_info.client_data=(void *) &error_manager;
+
   jpeg_create_compress(&jpeg_info);
   JPEGDestinationManager(&jpeg_info,image);
   jpeg_info.image_width=(unsigned int) image->columns;
   jpeg_info.image_height=(unsigned int) image->rows;
   jpeg_info.input_components=3;
   jpeg_info.in_color_space=JCS_RGB;
-  switch (image_info->colorspace)
+
+  /*
+    Set JPEG colorspace as per user request.
+  */
   {
-    case CMYKColorspace:
-    {
-      jpeg_info.input_components=4;
-      jpeg_info.in_color_space=JCS_CMYK;
-      (void) TransformColorspace(image,CMYKColorspace);
-      break;
-    }
-    case YCbCrColorspace:
-    {
-      jpeg_info.in_color_space=JCS_YCbCr;
-      (void) TransformColorspace(image,YCbCrColorspace);
-      break;
-    }
-    case GRAYColorspace:
-    case Rec601LumaColorspace:
-    case Rec709LumaColorspace:
-    {
-      jpeg_info.in_color_space=JCS_GRAYSCALE;
-      (void) TransformColorspace(image,image_info->colorspace);
-      break;
-    }
-    default:
-    {
-      if (image->colorspace == CMYKColorspace)
-        {
-          jpeg_info.input_components=4;
-          jpeg_info.in_color_space=JCS_CMYK;
-          break;
-        }
-      if (image->colorspace == YCbCrColorspace)
-        {
-          jpeg_info.in_color_space=JCS_YCbCr;
-          break;
-        }
-      (void) TransformColorspace(image,RGBColorspace);
-      break;
-    }
+    MagickBool
+      colorspace_set=MagickFalse;
+    
+    if (IsCMYKColorspace(image_info->colorspace))
+      {
+	jpeg_info.input_components=4;
+	jpeg_info.in_color_space=JCS_CMYK;
+	colorspace_set=MagickTrue;
+      }
+    else if (IsYCbCrColorspace(image_info->colorspace))
+      {
+	jpeg_info.input_components=3;
+	jpeg_info.in_color_space=JCS_YCbCr;
+	colorspace_set=MagickTrue;
+      }
+    else if (IsGrayColorspace(image_info->colorspace))
+      {
+	jpeg_info.input_components=1;
+	jpeg_info.in_color_space=JCS_GRAYSCALE;
+	colorspace_set=MagickTrue;
+      }
+
+    if (!colorspace_set)
+      {
+	if (IsCMYKColorspace(image->colorspace))
+	  {
+	    jpeg_info.input_components=4;
+	    jpeg_info.in_color_space=JCS_CMYK;
+	  }
+	else if (IsYCbCrColorspace(image->colorspace))
+	  {
+	    jpeg_info.input_components=3;
+	    jpeg_info.in_color_space=JCS_YCbCr;
+	  }
+	else if ((IsGrayColorspace(image->colorspace) ||
+		  (characteristics.grayscale)))
+	  {
+	    jpeg_info.input_components=1;
+	    jpeg_info.in_color_space=JCS_GRAYSCALE;
+	  }
+	else
+	  {
+	    jpeg_info.input_components=3;
+	    jpeg_info.in_color_space=JCS_RGB;
+	  }
+      }
   }
 
   input_colorspace=UndefinedColorspace;
@@ -1743,19 +1957,8 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
         }
     }
 
-
-
-  if ((image_info->type != TrueColorType) &&
-      (image_info->type != TrueColorMatteType) &&
-      (image_info->type != ColorSeparationType) &&
-      (image_info->type != ColorSeparationMatteType) &&
-      (image->colorspace != CMYKColorspace) &&
-      (characteristics.grayscale))
-    {
-      jpeg_info.input_components=1;
-      jpeg_info.in_color_space=JCS_GRAYSCALE;
-    }
   jpeg_set_defaults(&jpeg_info);
+
   /*
     Determine bit depth.
   */
@@ -1813,9 +2016,8 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
       *value;
 
     /*
-      Allow the user to select the DCD encoding algorithm.
+      Allow the user to select the DCT encoding algorithm.
     */
-    /* jpeg_info.dct_method=JDCT_DEFAULT; */
     if ((value=AccessDefinition(image_info,"jpeg","dct-method")))
       {
         if (LocaleCompare(value,"ISLOW") == 0)
@@ -1958,7 +2160,9 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
           jpeg_info.comp_info[i].v_samp_factor=1;
         }
     }
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Starting JPEG compression");
+
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Starting JPEG compression");
   jpeg_start_compress(&jpeg_info,True);
   if (image->logging)
     {
@@ -2102,9 +2306,12 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
         register long
           j;
 
-        if (LocaleNCompare(profile_name,"APP",3) != 0)
+        if (LocaleNCompare(profile_name,"APP",3) != 0 &&
+            LocaleNCompare(profile_name,"Exif",4) != 0)
           continue;
-        x=atol(profile_name+3);
+        x=1;
+        if (LocaleNCompare(profile_name,"APP",3) == 0)
+          x=MagickAtoL(profile_name+3);
         if (image->logging)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                                 "Profile: %s, %lu bytes",profile_name,
@@ -2132,9 +2339,10 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
     {
       if (jpeg_info.in_color_space == JCS_GRAYSCALE)
         {
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                                "Writing %d bit JCS_GRAYSCALE samples",
-                                jpeg_info.data_precision);
+	  if (image->logging)
+	    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+				  "Writing %d bit JCS_GRAYSCALE samples",
+				  jpeg_info.data_precision);
           for (y=0; y < (long) image->rows; y++)
             {
               p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -2169,9 +2377,10 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
         if ((jpeg_info.in_color_space == JCS_RGB) ||
             (jpeg_info.in_color_space == JCS_YCbCr))
           {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                                  "Writing %d bit JCS_RGB or JCS_YCbCr samples",
-                                  jpeg_info.data_precision);
+	    if (image->logging)
+	      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+				    "Writing %d bit JCS_RGB or JCS_YCbCr samples",
+				    jpeg_info.data_precision);
             for (y=0; y < (long) image->rows; y++)
               {
                 p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -2195,9 +2404,10 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
           }
         else
           {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                                  "Writing %d bit JCS_CMYK samples",
-                                  jpeg_info.data_precision);
+	    if (image->logging)
+	      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+				    "Writing %d bit JCS_CMYK samples",
+				    jpeg_info.data_precision);
             for (y=0; y < (long) image->rows; y++)
               {
                 p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -2227,9 +2437,10 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
   else
     if (jpeg_info.in_color_space == JCS_GRAYSCALE)
       {
-        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                              "Writing %d bit JCS_GRAYSCALE samples",
-                              jpeg_info.data_precision);
+	if (image->logging)
+	  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+				"Writing %d bit JCS_GRAYSCALE samples",
+				jpeg_info.data_precision);
         for (y=0; y < (long) image->rows; y++)
           {
             p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -2264,9 +2475,10 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
       if ((jpeg_info.in_color_space == JCS_RGB) ||
           (jpeg_info.in_color_space == JCS_YCbCr))
         {
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                                "Writing %d bit JCS_RGB or JCS_YCbCr samples",
-                                jpeg_info.data_precision);
+	  if (image->logging)
+	    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+				  "Writing %d bit JCS_RGB or JCS_YCbCr samples",
+				  jpeg_info.data_precision);
           for (y=0; y < (long) image->rows; y++)
             {
               p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -2290,9 +2502,10 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
         }
       else
         {
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                                "Writing %d bit JCS_CMYK samples",
-                                jpeg_info.data_precision);
+	  if (image->logging)
+	    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+				  "Writing %d bit JCS_CMYK samples",
+				  jpeg_info.data_precision);
           for (y=0; y < (long) image->rows; y++)
             {
               p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -2318,7 +2531,8 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
                   break;
             }
         }
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Finishing JPEG compression");
+  if (image->logging)
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Finishing JPEG compression");
   jpeg_finish_compress(&jpeg_info);
   /*
     Free memory.
