@@ -138,7 +138,7 @@ MagickExport Image *BorderImage(const Image *image,
 MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
                                ExceptionInfo *exception)
 {
-#define FrameImageText  "[%s] Add frame to image..."
+#define FrameImageText "[%s] Frame: %lux%lu%+ld%+ld bevel inner %ld outer %ld..."
 
   Image
     *frame_image;
@@ -290,14 +290,17 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   /*
     Draw sides of ornamental border.
   */
-#if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status) private (p, q, width, x)
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp parallel for schedule(static,8) shared(row_count, status) private (p, q, width, x)
 #endif
   for (y=0; y < (long) image->rows; y++)
     {
       MagickBool
         thread_status;
-    
+
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_FrameImage)
+#endif    
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -333,14 +336,19 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
           if (!SyncImagePixelsEx(frame_image,exception))
             thread_status=MagickFail;
         }
-#if defined(HAVE_OPENMP)
-#  pragma omp critical
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_FrameImage)
 #endif
       {
         row_count++;
         if (QuantumTick(row_count,image->rows))
           if (!MagickMonitorFormatted(row_count,image->rows,exception,
-                                      FrameImageText,image->filename))
+                                      FrameImageText,image->filename,
+				      frame_info->width,frame_info->height,
+				      frame_info->x,
+				      frame_info->y,
+				      frame_info->inner_bevel,
+				      frame_info->outer_bevel))
             thread_status=MagickFail;
           
         if (thread_status == MagickFail)
@@ -420,7 +428,7 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
 %  The format of the RaiseImage method is:
 %
 %      unsigned int RaiseImage(Image *image,const RectangleInfo *raise_info,
-%        const int raise)
+%        const int raise_flag)
 %
 %  A description of each parameter follows:
 %
@@ -428,20 +436,20 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
 %
 %    o raise_info: Define the width and height of the raise area.
 %
-%    o raise: A value other than zero creates a 3-D raise effect,
+%    o raise_flag: A value other than zero creates a 3-D raise effect,
 %      otherwise it has a lowered effect.
 %
 %
 */
-MagickExport MagickPassFail
-RaiseImage(Image *image,const RectangleInfo *raise_info,const int raise)
-{
 #define AccentuateFactor  (double) ScaleCharToQuantum(135)
 #define HighlightFactor  (double) ScaleCharToQuantum(190)
 #define ShadowFactor  (double) ScaleCharToQuantum(190)
-#define RaiseImageText  "[%s] Raise image..."
+#define RaiseImageText  "[%s] Raise..."
 #define TroughFactor  (double) ScaleCharToQuantum(135)
 
+MagickExport MagickPassFail
+RaiseImage(Image *image,const RectangleInfo *raise_info,const int raise_flag)
+{
   unsigned long
     row_count=0;
 
@@ -475,21 +483,24 @@ RaiseImage(Image *image,const RectangleInfo *raise_info,const int raise)
 
   foreground=MaxRGBDouble;
   background=0.0;
-  if (!raise)
+  if (!raise_flag)
     {
       foreground=0.0;
       background=MaxRGBDouble;
     }
   (void) SetImageType(image,TrueColorType);
 
-#if defined(HAVE_OPENMP)
-#  pragma omp parallel for shared(row_count, status) private (q, x)
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp parallel for schedule(static,4) shared(row_count, status) private (q, x)
 #endif
   for ( y=0; y < (long) image->rows; y++)
     {
       MagickBool
         thread_status;
-    
+
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_RaiseImage)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -585,8 +596,8 @@ RaiseImage(Image *image,const RectangleInfo *raise_info,const int raise)
           if (!SyncImagePixelsEx(image,&image->exception))
             thread_status=MagickFail;
         }
-#if defined(HAVE_OPENMP)
-#  pragma omp critical
+#if defined(HAVE_OPENMP) && !defined(DisableSlowOpenMP)
+#  pragma omp critical (GM_RaiseImage)
 #endif
       {
         row_count++;
