@@ -540,254 +540,6 @@ ColorMatrixImage(Image *image,const unsigned int order,const double *color_matri
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
-%     C o n v o l v e I m a g e                                               %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  ConvolveImage() applies a custom convolution kernel to the image.
-%
-%  The format of the ConvolveImage method is:
-%
-%      Image *ConvolveImage(const Image *image,const unsigned int order,
-%        const double *kernel,ExceptionInfo *exception)
-%
-%  A description of each parameter follows:
-%
-%    o image: The image.
-%
-%    o order: The number of columns and rows in the filter kernel.
-%
-%    o kernel: An array of double representing the convolution kernel.
-%
-%    o exception: Return any errors or warnings in this structure.
-%
-%
-*/
-#define ConvolveImageText "[%s] Convolve: order %u..."
-MagickExport Image *ConvolveImage(const Image *image,const unsigned int order,
-                                  const double *kernel,ExceptionInfo *exception)
-{
-  double
-    *normal_kernel;
-
-  Image
-    *convolve_image;
-
-  long
-    width,
-    y;
-
-  MagickPassFail
-    status;
-
-  const MagickBool
-    matte=((image->matte) || (image->colorspace == CMYKColorspace));
-
-  /*
-    Initialize convolve image attributes.
-  */
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  width=(long) order;
-  if ((width % 2) == 0)
-    ThrowImageException3(OptionError,UnableToConvolveImage,
-                         KernelWidthMustBeAnOddNumber);
-  if (((long) image->columns < width) || ((long) image->rows < width))
-    ThrowImageException3(OptionError,UnableToConvolveImage,
-                         ImageSmallerThanKernelWidth);
-  convolve_image=CloneImage(image,image->columns,image->rows,MagickTrue,exception);
-  if (convolve_image == (Image *) NULL)
-    return((Image *) NULL);
-  convolve_image->storage_class=DirectClass;
-  {
-    /*
-      Build normalized kernel.
-    */
-    double
-      normalize;
-    
-    register long
-      i;
-
-    normal_kernel=MagickAllocateMemory(double *,width*width*sizeof(double));
-    if (normal_kernel == (double *) NULL)
-      {
-        DestroyImage(convolve_image);
-        ThrowImageException(ResourceLimitError,MemoryAllocationFailed,
-                            MagickMsg(OptionError,UnableToConvolveImage));
-      }
-    normalize=0.0;
-    for (i=0; i < (width*width); i++)
-      normalize+=kernel[i];
-    if (AbsoluteValue(normalize) <= MagickEpsilon)
-      normalize=1.0;
-    normalize=1.0/normalize;
-    for (i=0; i < (width*width); i++)
-      {
-        normal_kernel[i]=normalize*kernel[i];
-      }
-  }
-  
-  if (LogMagickEvent(TransformEvent,GetMagickModule(),
-                     "  ConvolveImage with %ldx%ld kernel:",width,width))
-    {
-      /*
-        Log convolution matrix.
-      */
-      char
-        cell_text[MaxTextExtent],
-        row_text[MaxTextExtent];
-
-      const double
-        *k;
-
-      long
-        u,
-        v;
-
-      k=kernel;
-      for (v=0; v < width; v++)
-        {
-          *row_text='\0';
-          for (u=0; u < width; u++)
-            {
-              FormatString(cell_text,"%#12.4g",*k++);
-              (void) strlcat(row_text,cell_text,sizeof(row_text));
-              if (u%5 == 4)
-                {
-                  (void) LogMagickEvent(TransformEvent,GetMagickModule(),
-                                        "   %.64s", row_text);
-                  *row_text='\0';
-                }
-            }
-          if (u > 5)
-            (void) strlcat(row_text,"\n",sizeof(row_text));
-          if (row_text[0] != '\0')
-            (void) LogMagickEvent(TransformEvent,GetMagickModule(),
-                                  "   %s", row_text);
-        }
-    }
-
-  status=MagickPass;
-  /*
-    Convolve image.
-  */
-  {
-    unsigned long
-      row_count=0;
-
-    DoublePixelPacket
-      zero;
-
-    (void) memset(&zero,0,sizeof(DoublePixelPacket));
-#if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
-#endif
-    for (y=0; y < (long) convolve_image->rows; y++)
-      {
-        const PixelPacket
-          *p;
-    
-        PixelPacket
-          *q;
-
-        long
-          x;
-
-        MagickBool
-          thread_status;
-
-        thread_status=status;
-        if (thread_status == MagickFail)
-          continue;
-
-        p=AcquireImagePixels(image,-width/2,y-width/2,image->columns+width,width,
-                             exception);
-        q=SetImagePixelsEx(convolve_image,0,y,convolve_image->columns,1,exception);
-        if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
-          thread_status=MagickFail;
-
-        if (thread_status != MagickFail)
-          {
-            for (x=0; x < (long) convolve_image->columns; x++)
-              {
-                DoublePixelPacket
-                  pixel;
-
-                const PixelPacket
-                  *r;
-
-                long
-                  u,
-                  v;
-
-                const double
-                  *k;
-
-                r=p;
-                pixel=zero;
-                k=normal_kernel;
-                for (v=width; v > 0; v--)
-                  {
-                    for (u=0; u < width; u++)
-                      {
-                        pixel.red+=(*k)*r[u].red;
-                        pixel.green+=(*k)*r[u].green;
-                        pixel.blue+=(*k)*r[u].blue;
-                        if (matte)
-                          pixel.opacity+=(*k)*r[u].opacity;
-                        k++;
-                      }
-                    r+=image->columns+width;
-                  }
-                q->red=RoundDoubleToQuantum(pixel.red);
-                q->green=RoundDoubleToQuantum(pixel.green);
-                q->blue=RoundDoubleToQuantum(pixel.blue);
-                q->opacity=RoundDoubleToQuantum(pixel.opacity);
-                p++;
-                q++;
-              }
-            if (!SyncImagePixelsEx(convolve_image,exception))
-              thread_status=MagickFail;
-          }
-#if defined(HAVE_OPENMP)
-#  pragma omp critical (GM_ConvolveImage)
-#endif
-        {
-          row_count++;
-          if (QuantumTick(row_count,image->rows))
-            if (!MagickMonitorFormatted(row_count,image->rows,exception,
-                                        ConvolveImageText,
-                                        convolve_image->filename,
-					order))
-              thread_status=MagickFail;
-          
-          if (thread_status == MagickFail)
-            status=MagickFail;
-        }
-      }
-  }
-  MagickFreeMemory(normal_kernel);
-  if (MagickFail == status)
-    {
-      DestroyImage(convolve_image);
-      convolve_image=(Image *) NULL;
-    }
-  else
-    {
-      convolve_image->is_grayscale=image->is_grayscale;
-    }
-  return(convolve_image);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
 %     I m p l o d e I m a g e                                                 %
 %                                                                             %
 %                                                                             %
@@ -876,7 +628,7 @@ MagickExport Image *ImplodeImage(const Image *image,const double amount,
       row_count=0;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
+#  pragma omp parallel for schedule(static,4) shared(row_count, status)
 #endif
     for (y=0; y < (long) image->rows; y++)
       {
@@ -896,7 +648,10 @@ MagickExport Image *ImplodeImage(const Image *image,const double amount,
 
         MagickPassFail
           thread_status;
-        
+
+#if defined(HAVE_OPENMP)
+#  pragma omp critical (GM_ImplodeImage)
+#endif
         thread_status=status;
         if (thread_status == MagickFail)
           continue;
@@ -1224,7 +979,7 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
     Paint each row of the image.
   */
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
+#  pragma omp parallel for schedule(static,4) shared(row_count, status)
 #endif
   for (y=0; y < (long) image->rows; y++)
     {
@@ -1243,7 +998,10 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
 
       MagickBool
         thread_status;
-      
+
+#if defined(HAVE_OPENMP)
+#  pragma omp critical (GM_OilPaintImage)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -1758,7 +1516,7 @@ MagickExport Image *SwirlImage(const Image *image,double degrees,
       row_count=0;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(dynamic,8) shared(row_count, status)
+#  pragma omp parallel for schedule(static,8) shared(row_count, status)
 #endif
     for (y=0; y < (long) image->rows; y++)
       {
@@ -1778,7 +1536,10 @@ MagickExport Image *SwirlImage(const Image *image,double degrees,
 
         MagickPassFail
           thread_status;
-        
+
+#if defined(HAVE_OPENMP)
+#  pragma omp critical (GM_SwirlImage)
+#endif
         thread_status=status;
         if (thread_status == MagickFail)
           continue;
@@ -1948,7 +1709,7 @@ MagickExport Image *WaveImage(const Image *image,const double amplitude,
       row_count=0;
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
+#  pragma omp parallel for schedule(static,4) shared(row_count, status)
 #endif
     for (y=0; y < (long) wave_image->rows; y++)
       {
@@ -1963,7 +1724,10 @@ MagickExport Image *WaveImage(const Image *image,const double amplitude,
 
         MagickPassFail
           thread_status;
-        
+
+#if defined(HAVE_OPENMP)
+#  pragma omp critical (GM_WaveImage)
+#endif
         thread_status=status;
         if (thread_status == MagickFail)
           continue;
