@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003 - 2009 GraphicsMagick Group
+  Copyright (C) 2003 - 2010 GraphicsMagick Group
   Copyright (C) 2002 ImageMagick Studio
   Copyright 1991-1999 E. I. du Pont de Nemours and Company
  
@@ -185,16 +185,6 @@ typedef enum
   PseudoClass
 } ClassType;
 
-typedef enum
-{
-  UndefinedCompliance = 0x0000,
-  NoCompliance = 0x0000,
-  SVGCompliance = 0x0001,
-  X11Compliance = 0x0002,
-  XPMCompliance = 0x0004,
-  AllCompliance = 0xffff
-} ComplianceType;
-
 typedef enum
 {
   UndefinedCompositeOp = 0,
@@ -242,12 +232,17 @@ typedef enum
   NoCompression,
   BZipCompression,
   FaxCompression,
+  Group3Compression = FaxCompression,
   Group4Compression,
   JPEGCompression,
   LosslessJPEGCompression,
   LZWCompression,
   RLECompression,
-  ZipCompression
+  ZipCompression,
+  LZMACompression,              /* Lempel-Ziv-Markov chain algorithm */
+  JPEG2000Compression,          /* ISO/IEC std 15444-1 */
+  JBIG1Compression,             /* ISO/IEC std 11544 / ITU-T rec T.82 */
+  JBIG2Compression              /* ISO/IEC std 14492 / ITU-T rec T.88 */
 } CompressionType;
 
 typedef enum
@@ -535,29 +530,6 @@ typedef struct _PixelPacket
 #endif
 } PixelPacket;
 
-typedef struct _ColorInfo
-{
-  char
-    *path,
-    *name;
-
-  ComplianceType
-    compliance;
-
-  PixelPacket
-    color;
-
-  unsigned int
-    stealth;
-
-  unsigned long
-    signature;
-
-  struct _ColorInfo
-    *previous,
-    *next;
-} ColorInfo;
-
 typedef struct _DoublePixelPacket
 {
   double
@@ -567,6 +539,15 @@ typedef struct _DoublePixelPacket
     opacity;
 } DoublePixelPacket;
 
+typedef struct _FloatPixelPacket
+{
+  float
+    red,
+    green,
+    blue,
+    opacity;
+} FloatPixelPacket;
+
 /*
   ErrorInfo is used to record statistical difference (error)
   information based on computed Euclidean distance in RGB space.
@@ -671,16 +652,6 @@ typedef struct _SegmentInfo
     y2;
 } SegmentInfo;
 
-typedef struct _ImageCharacteristics
-{
-  MagickBool
-    cmyk,               /* CMYK(A) image */
-    grayscale,          /* Grayscale image */
-    monochrome,         /* Black/white image */
-    opaque,             /* Opaque image */
-    palette;            /* Colormapped image */
-} ImageCharacteristics;
-
 typedef struct _Image
 {
   ClassType
@@ -760,7 +731,7 @@ typedef struct _Image
     gravity;            /* Image placement gravity */
 
   CompositeOperator
-    compose;            /* Image placement composition */
+    compose;            /* Image placement composition (default OverCompositeOp) */
 
   DisposeType
     dispose;            /* GIF disposal option */
@@ -1028,9 +999,6 @@ extern MagickExport Image
 extern MagickExport ImageInfo
   *CloneImageInfo(const ImageInfo *);
 
-extern MagickExport ImageType
-  GetImageType(const Image *,ExceptionInfo *);
-
 extern MagickExport const char
   *AccessDefinition(const ImageInfo *image_info,const char *magick,
      const char *key);
@@ -1039,9 +1007,6 @@ extern MagickExport int
   GetImageGeometry(const Image *,const char *,const unsigned int,
   RectangleInfo *);
 
-extern MagickExport RectangleInfo
-  GetImageBoundingBox(const Image *,ExceptionInfo *exception);
-
 /* Functions which return unsigned int as a True/False boolean value */
 extern MagickExport MagickBool
   IsTaintImage(const Image *),
@@ -1052,21 +1017,19 @@ extern MagickExport MagickPassFail
   AddDefinitions(ImageInfo *image_info,const char *options,
     ExceptionInfo *exception),
   AnimateImages(const ImageInfo *image_info,Image *image),
-  ClipImage(Image *),
+  ClipImage(Image *image),
   ClipPathImage(Image *image,const char *pathname,const MagickBool inside),
   DisplayImages(const ImageInfo *image_info,Image *image),
-  GetImageCharacteristics(const Image *image,ImageCharacteristics *characteristics,
-    const MagickBool optimize,ExceptionInfo *exception),
   RemoveDefinitions(const ImageInfo *image_info,const char *options),
-  SetImage(Image *,const Quantum),
+  SetImage(Image *image,const Quantum),
+  SetImageColor(Image *image,const PixelPacket *pixel),
+  SetImageColorRegion(Image *image,long x,long y,unsigned long width,
+		      unsigned long height,const PixelPacket *pixel),
   SetImageClipMask(Image *image,const Image *clip_mask),
-  SetImageDepth(Image *,const unsigned long),
-  SetImageInfo(ImageInfo *,const MagickBool,ExceptionInfo *),
-  SetImageType(Image *,const ImageType),
-  SyncImage(Image *);
-
-extern MagickExport unsigned long
-  GetImageDepth(const Image *,ExceptionInfo *);
+  SetImageDepth(Image *image,const unsigned long),
+  SetImageInfo(ImageInfo *image_info,const unsigned int flags,ExceptionInfo *exception),
+  SetImageType(Image *image,const ImageType),
+  SyncImage(Image *image);
 
 extern MagickExport void
   AllocateNextImage(const ImageInfo *,Image *),
@@ -1077,8 +1040,27 @@ extern MagickExport void
   ModifyImage(Image **,ExceptionInfo *),
   SetImageOpacity(Image *,const unsigned int);
 
+#if defined(MAGICK_IMPLEMENTATION)
+  /*
+    SetImageInfo flags specification.
+  */
+#  define SETMAGICK_FALSE    0x00000 /* MagickFalse ("read") */
+#  define SETMAGICK_TRUE     0x00001 /* MagickTrue ("write+rectify") */
+#  define SETMAGICK_READ     0x00002 /* Filespec will be read */
+#  define SETMAGICK_WRITE    0x00004 /* Filespec will be written */
+#  define SETMAGICK_RECTIFY  0x00008 /* Look for adjoin in filespec */
+#endif /* defined(MAGICK_IMPLEMENTATION) */
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
 
 #endif /* _MAGICK_IMAGE_H */
+
+/*
+ * Local Variables:
+ * mode: c
+ * c-basic-offset: 2
+ * fill-column: 78
+ * End:
+ */
