@@ -40,6 +40,8 @@ extern "C" {
 #define HAVE_VSNPRINTF 1
 #define HAVE_TEMPNAM 1
 #define HAVE_RAISE 1
+#define HAVE_SPAWNVP 1
+#define HAVE_PROCESS_H 1
 
 /*
   libtiff features.
@@ -71,6 +73,12 @@ extern "C" {
 #  define inline __inline
 #endif
 
+// Visual C++ does not usually seem to support the C'99 restrict keyword
+// Maybe it will be added in some version.
+#if defined(_VISUALC_)  /* && (_MSC_VER <= 1500) */
+#  define restrict /* nothing */
+#endif
+
 #if !defined(chsize)
 # if defined(__BORLANDC__)
 #   define chsize(file,length) chsize(file,length)
@@ -99,10 +107,14 @@ extern "C" {
 */
 #define HAVE_POPEN 1
 #define HAVE__POPEN 1
-#define popen(command,mode)  _popen(command,mode)
+#if !defined(popen)
+#  define popen(command,mode)  _popen(command,mode)
+#endif /* !defined(popen) */
 #define HAVE_PCLOSE 1
 #define HAVE__PCLOSE 1
-#define pclose(stream)  _pclose(stream)
+#if !defined(pclose)
+#  define pclose(stream)  _pclose(stream)
+#endif /* !defined(pclose) */
 
 /*
   Visual C++ 7.0 supports GlobalMemoryStatusEx but MinGW headers and
@@ -334,3 +346,11 @@ extern MagickExport long
 #endif /* !C++ */
 
 #endif /* !_MAGICK_NTBASE_H */
+
+/*
+ * Local Variables:
+ * mode: c
+ * c-basic-offset: 2
+ * fill-column: 78
+ * End:
+ */
