@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2004, 2008 GraphicsMagick Group
+% Copyright (C) 2004-2010 GraphicsMagick Group
 %
 % This program is covered by multiple licenses, which are described in
 % Copyright.txt. You should have received a copy of Copyright.txt with this
@@ -193,11 +193,14 @@ PixelIterateMonoRead(PixelIteratorMonoReadCallback call_back,
         thread_status;
 
       const PixelPacket
-        *pixels;
+        * restrict pixels;
 
       const IndexPacket
-        *indexes;
+        * restrict indexes;
 
+#if defined(HAVE_OPENMP)
+#  pragma omp critical (GM_PixelIterateMonoRead)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -324,11 +327,14 @@ PixelIterateMonoModify(PixelIteratorMonoModifyCallback call_back,
         thread_status;
 
       PixelPacket
-        *pixels;
+        * restrict pixels;
 
       IndexPacket
-        *indexes;
+        * restrict indexes;
 
+#if defined(HAVE_OPENMP)
+#  pragma omp critical (GM_PixelIterateMonoModify)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -475,13 +481,16 @@ PixelIterateDualRead(PixelIteratorDualReadCallback call_back,
         second_row;
 
       const PixelPacket
-        *first_pixels,
-        *second_pixels;
+        * restrict first_pixels,
+        * restrict second_pixels;
 
       const IndexPacket
-        *first_indexes,
-        *second_indexes;
+        * restrict first_indexes,
+        * restrict second_indexes;
 
+#if defined(HAVE_OPENMP)
+#  pragma omp critical (GM_PixelIterateDualRead)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -635,21 +644,24 @@ PixelIterateDualImplementation(PixelIteratorDualModifyCallback call_back,
         thread_status;
 
       const PixelPacket
-        *source_pixels;
+        * restrict source_pixels;
 
       const IndexPacket
-        *source_indexes;
+        * restrict source_indexes;
 
       PixelPacket
-        *update_pixels;
+        * restrict update_pixels;
 
       IndexPacket
-        *update_indexes;
+        * restrict update_indexes;
 
       register long
         source_row,
         update_row;
 
+#if defined(HAVE_OPENMP)
+#  pragma omp critical (GM_PixelIterateDualImplementation)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -931,23 +943,26 @@ PixelIterateTripleImplementation(PixelIteratorTripleModifyCallback call_back,
         thread_status;
 
       const PixelPacket
-        *source1_pixels,
-        *source2_pixels;
+        * restrict source1_pixels,
+        * restrict source2_pixels;
 
       const IndexPacket
-        *source1_indexes,
-        *source2_indexes;
+        * restrict source1_indexes,
+        * restrict source2_indexes;
 
       PixelPacket
-        *update_pixels;
+        * restrict update_pixels;
 
       IndexPacket
-        *update_indexes;
+        * restrict update_indexes;
 
       long
         source_row,
         update_row;
 
+#if defined(HAVE_OPENMP)
+#  pragma omp critical (GM_PixelIterateTripleImplementation)
+#endif
       thread_status=status;
       if (thread_status == MagickFail)
         continue;
@@ -990,14 +1005,15 @@ PixelIterateTripleImplementation(PixelIteratorTripleModifyCallback call_back,
       update_indexes=AccessMutableIndexes(update_image);
 
       if (thread_status != MagickFail)
-        status=(call_back)(mutable_data,immutable_data,
-                           source1_image,source1_pixels,source1_indexes,
-                           source2_image,source2_pixels,source2_indexes,
-                           update_image,update_pixels,update_indexes,
-                           columns,exception);
-      
-      if (!SyncImagePixelsEx(update_image,exception))
-        thread_status=MagickFail;
+        thread_status=(call_back)(mutable_data,immutable_data,
+				  source1_image,source1_pixels,source1_indexes,
+				  source2_image,source2_pixels,source2_indexes,
+				  update_image,update_pixels,update_indexes,
+				  columns,exception);
+
+      if (thread_status != MagickFail)
+	if (!SyncImagePixelsEx(update_image,exception))
+	  thread_status=MagickFail;
 
 #if defined(HAVE_OPENMP)
 #  pragma omp critical (GM_PixelIterateTripleImplementation)
