@@ -220,7 +220,7 @@ extern "C" {
 /*
   OpenMP support requires version 2.0 (March 2002) or later.
 */
-#if defined(_OPENMP) && (_OPENMP >= 200203)
+#if defined(_OPENMP) && ((_OPENMP >= 200203) || defined(__OPEN64__))
 #  include <omp.h>
 #  define HAVE_OPENMP 1
 #endif
@@ -394,7 +394,7 @@ extern int vsnprintf(char *s, size_t n, const char *format, va_list ap);
 #  define MagickTell(fildes) /* __int64 */ _telli64(fildes)
 #else
 #  define MagickSeek(fildes,offset,whence)  lseek(fildes,offset,whence)
-#  define MagickTell(fildes) tell(filedes)
+#  define MagickTell(fildes) (MagickSeek(fildes,0,SEEK_CUR))
 #endif
 
 #if defined(MSWINDOWS) && !defined(Windows95) && !defined(__BORLANDC__) && \
@@ -477,3 +477,11 @@ extern MagickExport const unsigned long
 #endif /* defined(__cplusplus) || defined(c_plusplus) */
 
 #endif /* ifndef _MAGICK_STUDIO_H */
+
+/*
+ * Local Variables:
+ * mode: c
+ * c-basic-offset: 2
+ * fill-column: 78
+ * End:
+ */
